% AnalyzaHRV.m
%
%   AnalyzaHRV.m je obslun aplikace pro zobrazen a zpracovn zaznamenanch dat
%   EKG. Program se spout volnm jeho nzvu bez parametr. Program
%   vyhled v pipraven sloce soubory s pponou ".mat". Tyto soubory
%   zskme importem z CSV soubor z programu BSL Lessons. Soubory
%   obsahujc v nzvu etezec "pred" jsou povazovny za hodnoty EKG ped
%   zt, soubory obsahujc v nzvu etzec "po" pak za hodnoty EKG po
%   zti. Pokud je nastaveno batchmode=0 je teba nast soubor kliknutm ve
%   sloce PrevedeneHodnoty.
%   Ped sputnm programu je teba v sekci konfigurace nastavit
%   vzorkovac frekvenci pouitou pro zznam dat. Dle je zde 
%   parametr ovlivujc bh aplipkace:
%       batchmode - pokud je 1 pak program prohled zvolenou sloku a
%       zpracuje vechny soubroy v tto sloce umstn
%                 - pokud je 0 pak se pedpokld e je v pamti ji nahrn
%                 vektor hodnot zznamu EKG pojmenovan Data
%   Vstupem programu je pole hodnot obsahujc stedn hodnotu RR
%   intervalu, pole chybn detekovanch RR interval
%   sloky spektrlnho vkonu LF a HF a jejich vzjemn pomr LF/HF.

% -------------------------------------------------------------------
% Nastaven parametr

%################
%Chovn programu
%################
batchmode = 0;          %dvkov zpracovn soubor ve sloce
verbosity = 4;
%###################
%Pojmenovn soubor
%###################

fileMask = {'*pred*.mat','*po*.mat'};
dataDirectory = 'PrevedeneHodnoty';

%###################
%Parametry zpracovn
%###################

Fs = 1000;              %Vzorkovac frekvence [Hz]
orezPred = 3;           %Odebrn signlu na zatku [sec]
orezZa = 3;             %Odebrn signlu na konci [sec]

detektor = 'b';         % Typ QRS detektoru a = zkladn, prahovac
                        %                   b = pokroily QRS
RLevelA = 1 ;           % Prh pro detekci QRS detektoru                    
RLevelB = 0.1 ;
%###################
%Vstupn filtr
%###################

firN = 200;             %d filtru
firLf =10;             %Spodn frekvence psmov propusti
firHf = 20;             %Horn frekvence psmov propusti

%###################
%Kontrola deketovanch R vln
%###################
difTmax = 70            % Maximln vzdlenost sousednch R vln [%]
difAmax = 70            % Maximln zmna amplitudy sousednch R vln [%]


%###################
%Vpoet PSD
%###################
%Psma v [Hz]
VLF = [0,0.04];
LF = [0.04,0.15];
HF = [0.15,0.4];
fs2 = 2;                 % Frekvence pevzorkovn [Hz]
fPoints = 1048;         % Poet bod analzy periodogramu
Normalize = 1;          % Vpoet normalizovanho PSD

%###################
%Zobrazeni graf v single file modu ( 1 = zobrazit, 0 = nezobrazit)
%###################
% Graf 1 signl ped a po filtraci vetn spekter
Graf1 = 1;
% Graf 2 frekvenn a fzov charakteristika fir filtru
Graf2 = 1;
% Graf 3 zobrazen vsledk pokroilho QRS detektoru
Graf3 = 1;
% Graf 4 zobrazen filtrovanch RR - interval
Graf4 = 1;
% Graf 5 zobrazen vkonovho spektra hrv s vyznaenm VLF, LF a HF
Graf5 = 1;




%--------------------------------------------------------------------

% Prohled aktuln sloku a vyhled soubory se vstupnmi daty
if (batchmode == 1)
    files = '';
    for i = 1:size(fileMask,2)
    files = [files;dir(fullfile(dataDirectory,fileMask{i}))]
    end;
    fileCount = size(files,1);
    if (verbosity == 4)
        disp([cputime,' Ve sloce:', dataDirectory, ' nalezeno ', int2str(fileCount) ,' soubor.']); 
    end
else
    fileCount = 1
end;


% Nvrh filtru
h=fir1(firN,[firLf firHf]/(Fs/2));


for fileNr = 1:fileCount
    if (batchmode == 1)
        if (verbosity > 2)
            disp([cputime,' Zpracovvm: ', files(fileNr).name;]); 
        end;
        load(fullfile(dataDirectory,files(fileNr).name));
    end;
    
    %Separace dat ekg do samostan promnn s oznutm
    orezPredVzorky = floor(Fs*orezPred);
    orezZaVzorky = floor(Fs*orezZa);
    ecg_in = data(orezPredVzorky:end-orezZaVzorky,2);
    

%Filtrace signlu
    ecg_filtred = filter (h,1,ecg_in);
    %Graf 1
    if( batchmode == 0 && Graf1 == 1)
        Graph_t = linspace(0,1/(60*Fs)*length(ecg_in),length(ecg_in));
        figure(1);
        subplot(2,1,1)
        plot(Graph_t,ecg_in,'g');
        hold on;
        plot(Graph_t,ecg_filtred,'r');
        title('Signl ped (zelen) a po fitraci (erven)')
        ylabel('Amp [mV]')
        xlabel('t [min]')
        hold off;

        subplot(2,1,2);
        spect_pred = fft(ecg_in);
        spect_po = fft(ecg_filtred);
        N = 300000;
        freq = Fs * (0 : N/2) / N;
        plot(freq,2/N * abs(spect_pred(1:N/2+1)),'g');
        hold on;
        plot(freq,2/N * abs(spect_po(1:N/2+1)),'r');
        title('Spektrum signlu ped (zelen) a po fitraci (erven)')
        ylabel('Amp [mV]')
        xlabel('[Hz]')
        axis([0 60 0 0.03])
        hold off;
    end;    
    %Graf 2
    if( batchmode == 0 && Graf2 == 1)
        figure(2);
        freqz(h);
    end;    
    
    if( batchmode == 1)
      Graf3 = 0;
    end;  
    
    if( detektor =='a')
        [hrv,casy,RR_times,RR_amp] = umocneni(ecg_filtred,RLevelA,Fs);
        R_t=RR_times;
    end;
    if (detektor == 'b')
       [hrv,resp, R_t, R_amp, R_index, S_t, S_amp]  = rpeakdetect(ecg_filtred, Fs, RLevelB, Graf3);
        RR_times = R_index;
        RR_amp = R_amp - S_amp;     %RS
    end;
    RR_intervals = diff(RR_times) / Fs;  
    RR_beats = 60./RR_intervals ;

% Kontrola a filtrovn detekovanch RR
    hrv_cleaned = clean_RR_RS([RR_times(2:end)',RR_intervals',RR_amp(2:end)'],difTmax,difAmax);
    pocetChyb = length(hrv) - length(hrv_cleaned);
    
% Interpolace chybjcch RR interval
    hrv_interpoled = RR_interpolation(hrv_cleaned,difTmax);
    pocetInterpolaci = length(hrv_interpoled) - length(hrv_cleaned)+1;
    %Graf 4
    if( batchmode == 0 && Graf4 == 1)
        figure(4);
        plot(RR_times(2:end)/Fs, RR_intervals,'+r'); 
        hold on;
        plot(hrv_interpoled(:,1)/Fs,hrv_interpoled(:,2),'+g');
        plot(hrv_cleaned(:,1)/Fs,hrv_cleaned(:,2),'+--b');
        title('RR vlny - R vlny odstrann filtrac jsou oznaeny erven')
        ylabel('RR [s]')
        xlabel('t [s]')
        axis([0 400 0.5 1.2]);
        hold off;
    end;  
% Pevod na s    
    R_t = hrv_interpoled(:,1)'/Fs;
% Prevod na ms
    RR_amp = hrv_interpoled(:,2)'*1000;
% Odstrann trendu
    RR_amp=detrend(RR_amp,'linear');
% Odstrann driftu    
    RR_amp=RR_amp-mean(RR_amp);
    
% Vpoet PSD

    maxF=fs2/2;  %Maximln frekvence - Nyquistova frekvence [Hz] - 
    dF=maxF/fPoints; % increment frekvenc
    F = linspace(0.0,maxF-dF,fPoints); % ada frekvenc pro vpoet PSD
%Vpoet sloek PSD pomoc Lomb - Scargle periodogramu
    PSD=lomb2(R_t',RR_amp',F,false);
    
%Vpoet spektrlnho vkonu
    output=calcAreas(F,PSD,VLF,LF,HF,Normalize);
    if (batchmode == 1)
        output.filename = files(fileNr).name;
    end;
    output.RpeakDetected = length(hrv);
    output.RpeakErrors = pocetChyb;
    
    if( batchmode == 0 && Graf5 == 1)
        figure(5);
        h1=subplot(1,1,1);
        plotPSD(h1,F,PSD,VLF,LF,HF,[0 0.6],[]);
        title('Vkonov spektrum pomoc Lomb-Scargle periodogramu');
    ylabel('PSD [ms2]')
    xlabel('f [Hz]')
    end;
   
       disp([output]);
       if (detektor=='a')
           hrv2=length(hrv);
       end;
       if (detektor=='b')
           hrv2=length(hrv);
       end;
       disp(['Pocet detekovanych R vln:',int2str(hrv2)]);
       disp(['Pocet chybne detekovanych R vln: ',int2str(pocetChyb)]);
       disp(['Pocet interpolovanych R vln: ',int2str(pocetInterpolaci)]);
       
       hrv_vycistene=length(hrv_cleaned);
       %disp(int2str(hrv_vycistene));
    %
    if (batchmode == 1)
        outstruct(fileNr) = output;
        output = '';
    end;    
end;

       