function [hrv] = RR_interpolation(temp_hrv, pc)

% Funkce pro vyhodnoceni kvality a procisteni detekovanych RR intervalu
% Kontroly:
% 1) Kontrola intervalu mezi RR intervaly - vychazi z fyziologie myokardu,
% kdy mezi bezprostredne nasledujicimi intervaly nemuze dojit ke zmene
% vetsi nez je zadano jako parametr pc
% 2) Kontrola rozdilu amplitudu R piku a S vlny - opet z fyziologie myokardu
% muze dojit ke zmene maximalne o hodnotu zadanou vstupnim parametrem pc2
%
% Pro zachovani moznosti identifikace potlacene R vlny po odstraneni piku
% mimo rozsah se po kazdem odstraneni piku snizi mez pro detekci o 3 %
%------------------------------------------------------------------------
%Vstupni hodnoty:
% [temp_hrv] - vstupni matice dat - organizace ve sloupcich:
%                   1. sloupec - casova osa - casy vyskytu R piku                                
%                   2. sloupec - RR intervaly - casy mezi sousednimi R piky 
%                   3. sloupec - RS amplitudy - rozdil amplitud R piku a S
%                   vlny
% [pc] - hranice ve ktere se muze pohybovat sousedici RR interval
%       Dle:
% Clifford G.D., McSharry P.E., Tarassenko L.: "Characterizing 
% Artefact in the Normal Human 24-Hour RR Time Series to Aid Identification 
% and Artificial Replication of Circadian Variations in Human Beat to Beat
% Heart Rate Using a Simple Threshold", Computers in Cardiology, 2002.
%     je tato hodnota cca 80% tedy sousedn intervaly se mohou zmenit o max
%     20 %. Tato hodnota je nastavena jako vychozi
%
%[pc2] - hranice o kterou se muze zmenit rozdil amplitud sousednich RS
%komplexu. Dle vyse uvedene literatury se tento rozdil pohybuje okolo 30%.
%Tato hodnota je take nastavena jako vychozi


%-------------------------------------------------------------------------
%Nastaveni vychozich hodnot


if nargin < 2
   pc=80;
end



% Inicializace
j=1;

% Vyhledani prvniho vzorku a overeni vstupnich dat
% Stredni hodnota RR intervalu
mean_tmp=mean(temp_hrv(:,2)); 
stp=0;             % inicializace stp == 0 platn data
st=size(temp_hrv); %
if st(1)>1         % number of rows must be > 1
    while ((stp==0) & (abs(temp_hrv(j,2)-mean_tmp)>10 )&(abs(temp_hrv(j,2)-temp_hrv(j+1,2)) > 10))
        j=j+1;
        if j==st(1)% 
            stp=1; % nastavi stp == 1 - vstupni data nelze pouzit
        end;       %
    end
end;
hrv=[];
if stp==0

%% prvni rr interval je povazovan za spravny, nasledujici intervaly
%% porovnavame podle nej
i=1;
hrv(i,1)=temp_hrv(j,1);
hrv(i,2)=temp_hrv(j,2);
hrv(i,3)=temp_hrv(j,3);
valid_RS = temp_hrv(j,3);
valid_RR = temp_hrv(j,2);
valid_T  = temp_hrv(j,1);
sz=length(temp_hrv);                %pocet vzorku na vstupu

% a bodge to allow the loop to run over the end 
%(mirror around the last sample by one sample)
temp_hrv(sz+1,3)=temp_hrv(sz-1,3);
temp_hrv(sz+1,2)=temp_hrv(sz-1,2);
% and add the correct time stamp - last time stamp plus new RR value
temp_hrv(sz+1,1)=temp_hrv(sz,2)+temp_hrv(sz+1,2);

% set hrv counter.
k=2;
for i=1:sz-j; % count through all data points starting from 
              % one after the first valid one
              % and keep only if it doesn't change by more than pc percent
              % on the last beat or on the next beat                 
% sorry - read this if you can
if (k < 3)
    trig_timing = (1+1-pc/100)*(temp_hrv(j+i,1)-temp_hrv(j+i-1,1));
else
    trig_timing = (1+1-pc/100)*(hrv(k-1,1)-hrv(k-2,1));
end    

tt_dif_before = temp_hrv(j+i,1) - temp_hrv(j+i-1,1);
tmp = abs(temp_hrv(j+i,1)-temp_hrv(j+i+1,1));
if(temp_hrv(j+i,1) > 214*1000)
    
    tmp = abs(temp_hrv(j+i,1)-temp_hrv(j+i+1,1));
end    
if(abs(temp_hrv(j+i,1)-temp_hrv(j+i+1,1)) < trig_timing)
       hrv(k,1)=temp_hrv(j+i,1);
       hrv(k,2)=temp_hrv(j+i,2);
       hrv(k,3)=temp_hrv(j+i,3);
       k=k+1; % counter for each beat kept
else
    tt_dif_interpol = floor((temp_hrv(j+i+1,1)-temp_hrv(j+i,1))/tt_dif_before);
    rr_dif_interpol = (temp_hrv(j+i+1,2) - temp_hrv(j+i,2))/tt_dif_interpol;
    rr_amp_interpol = (temp_hrv(j+i+1,3) - temp_hrv(j+i,3))/tt_dif_interpol;
    for m=0:tt_dif_interpol-1
       hrv(k,1)=temp_hrv(j+i,1)+floor(m*(temp_hrv(j+i+1,1)-temp_hrv(j+i,1))/tt_dif_interpol);;
       hrv(k,2)=temp_hrv(j+i,2)+ m*rr_dif_interpol;
       hrv(k,3)=temp_hrv(j+i,3)+ m*rr_amp_interpol;
       k=k+1; % counter for each beat kept
    end
end
end
end;