function [hrv] = clean_RR_RS(temp_hrv, pc, pc2)

% Funkce pro vyhodnoceni kvality a procisteni detekovanych RR intervalu
% Kontroly:
% 1) Kontrola intervalu mezi RR intervaly - vychazi z fyziologie myokardu,
% kdy mezi bezprostredne nasledujicimi intervaly nemuze dojit ke zmene
% vetsi nez je zadano jako parametr pc
% 2) Kontrola rozdilu amplitudy R piku a S vlny - opet z fyziologie myokardu
% muze dojit ke zmene maximalne o hodnotu zadanou vstupnim paramterem pc2
%
% Pro zachovani moznosti identifikace potlacene R vlny po odstraneni piku
% mimo rozsah se po kazdem odstraneni piku snizi mez pro detekci o 3 %
%------------------------------------------------------------------------
%Vstupni hodnoty:
% [temp_hrv] - vstupni matice dat - organizace ve sloupcich:
%                   1. sloupec - casova osa - casy vyskytu R piku                                
%                   2. sloupec - RR intervaly - casy mezi sousednimi R piky 
%                   3. sloupec - RS amplitudy - rozdil amplitud R piku a S
%                   vlny
% [pc] - hranice ve ktere se muze pohybovat sousedici RR interval
%       Dle:
% Clifford G.D., McSharry P.E., Tarassenko L.: "Characterizing 
% Artefact in the Normal Human 24-Hour RR Time Series to Aid Identification 
% and Artificial Replication of Circadian Variations in Human Beat to Beat
% Heart Rate Using a Simple Threshold", Computers in Cardiology, 2002.
%     je tato hodnota cca 80% tedy sousedn intervaly se mohou zmenit o max
%     20 %. Tato hodnota je nastavena jako vychozi
%
%[pc2] - hranice o kterou se muze zmenit rozdil amplitud sousednich QRS
%komplexu. Dle vyse uvedene literatury se tento rozdil pohybuje okolo 30%.
%Tato hodnota je take nastavena jako vychozi


%-------------------------------------------------------------------------

% Inicializace
j=1;
k=2;
l=0;

% Vyhledani prvniho vzorku a overeni vstupnich dat
% Stredni hodnota RR intervalu
mean_tmp=mean(temp_hrv(:,2)); 
stp=0;             % inicializace stp == 0 platna data
st=size(temp_hrv); %
if st(1)>1         % pocet radku musi byt > 1
    while ((stp==0) & (abs(temp_hrv(j,2)-mean_tmp)>10 )&(abs(temp_hrv(j,2)-temp_hrv(j+1,2)) > 10))
        j=j+1;
        if j==st(1)% 
            stp=1; % nastavi stp == 1 - vstupni data nelze pouzit
        end;       %
    end
end;
hrv=[];
if stp==0

%% prvni rr interval je povazovan za spravny, nasledujici intervaly
%% porovnavame podle nej
i=1;
hrv(i,1)=temp_hrv(j,1);
hrv(i,2)=temp_hrv(j,2);
hrv(i,3)=temp_hrv(j,3);
valid_RS = temp_hrv(j,3);
valid_RR = temp_hrv(j,2);
valid_T  = temp_hrv(j,1);
sz=length(temp_hrv);                %pocet vzorku na vstupu

% rozsireni sady dat okolo posledniho vzorku, tak aby jej bylo mozne
% zkontrolovat
temp_hrv(sz+1,3)=temp_hrv(sz-1,3);
temp_hrv(sz+1,2)=temp_hrv(sz-1,2);
% doplneni chybejiciho casu po poslednim vzorku
temp_hrv(sz+1,1)=temp_hrv(sz,2)+temp_hrv(sz+1,2);

for i=1:sz-j; % Cyklus prochazejici jednotlive vzorky
    trig_timing = ((1-( (pc-(l*3))/100))*(60/hrv(k-1,2))); % Vpoet mezni hodnoty rozdilu casu 
                                            % + snizeni citlivosti po predchozich vadnych datech
    rr_current = (60/temp_hrv(j+i,2)); %Hodnota aktualniho rr intervalu
    rr_before = (60/hrv(k-1,2));       %Hodnota predchoziho rr intervalu
    rr_next = (60/temp_hrv(j+i+1,2));  %Hodnota nasledujiciho rr intervalu
        
    trig_amp = abs((1-( (pc2-(l*3))/100))*(hrv(k-1,3))); % Vpoet mezn hodnoty zmny amplitudy
    rr_amp_current = temp_hrv(j+i,3);   %Amplituda aktualniho vzorku r vlny
    rr_amp_before = hrv(k-1,3);             %Amplituda predchozi r vlny
    rr_amp_next = temp_hrv(j+i+1,3);        %Amplituda nasledujici r vlny
    %!!!!! Vlastni kontrola vzorku    
    if (  (abs(rr_current-rr_before) < trig_timing) ...  %Rozdil casu aktualniho a predchoziho vzorku 
            && (abs(rr_current-rr_next) < trig_timing) ...  %Rozdil casu aktualniho a nasledujicih vzorku
            && ( abs(rr_amp_current-rr_amp_before)< trig_amp) ... %Zmena apmlitudy aktualniho a predchoziho vzorku
            && (abs(rr_amp_current -rr_amp_next) < trig_amp) ...  %Zmena amplitudy aktualniho a nasledujiciho vzorku
        )  
     %Vzorek splnuje podminky
     % Ulozeni do vystupu
       hrv(k,1)=temp_hrv(j+i,1); % cas
       hrv(k,2)=temp_hrv(j+i,2); % rr-interval
       hrv(k,3)=temp_hrv(j+i,3); % amplituda
       k=k+1; 
       l=0; % vynulovani kompenzace zmeny dat
 else
       l=l+1; % kompenzace zmeny dat - snizeni citlivosti pro dalsi vzorky
      end
end
end;