function [hrv,resp, R_t, R_amp, R_index, S_t, S_amp]  = rpeakdetect(data,samp_freq,thresh,testmode)

% [hrv, R_t, R_amp, R_index, S_t, S_amp]  = rpeakdetect(data, samp_freq, thresh, testmode); 
%--------------------------------------------------------------------------
%Vstupn hodnoty:
% [data] - nacteni dat ekg ve sloupcich ci radcich, vlozit muzeme jak s
% casovou osou tak bez ni
% [samp_freq] - vzorkovaci frekvence v Hz
% [tresh] - mez nad kterou jsou piky vyhodnoceny jak R pik ( v % /100) -
% vchoz hodnota 0,2 = 20%
% [testmode] - logick hodnota potlaceni zobrazeni grafu po detekci 
%     - vychozi hodnota 0
%--------------------------------------------------------------------------
% Vstupn hodnoty:
% [hrv] - variabilita hrv 
% [resp] - velikost prechodu mezi R pikem a S vlnou
% [R_t] - vektor detekovanch pk - asy vskytu
% [R_amp] - vektor detekovanch pk - amplitudy
% [R_index] - vektor detekovanch pk - indexy vskytu
% [S_t] - vektor detekovanch S-vln - asy vskytu
% [S_amp] - vektor detekovanch S-vln - amplitudy
%--------------------------------------------------------------------------
% Detektor napsn a upraven podle postup v nsledujch zdrojch: 
% J. Pan \& W. Tompkins - A real-time QRS detection algorithm 
% IEEE Transactions on Biomedical Engineering, vol. BME-32 NO. 3. 1985.
% P. Hamilton \& W. Tompkins. Quantitative Investigation of QRS 
% Detection  Rules Using the MIT/BIH Arrythmia Database. 
% IEEE Transactions on Biomedical Engineering, vol. BME-33, NO. 12.1986.
% 

%--------------------------------------------------------------------------
% Doplneni vychozich hodnot pokud nejsou zadany:

% Zobrazeni grafu
if nargin < 4
   testmode = 0;
end
% Vchoz mez
if nargin < 3
   thresh = 0.2;
end
%--------------------------------------------------------------------------

% Kontrola formtu vstupnch dat 
[a b] = size(data);         %velikost matice vstupnich dat
if(a>b)
 len =a;
end
if(b>a)
 len =b;                    % len - delka vstupnich dat
end

%--------------------------------------------------------------------------
%Doplneni casove osy pokud neni soucasti vstupnich dat
if (a || b == 1);
  tt = 1/samp_freq:1/samp_freq:ceil(len/samp_freq); % vytvoreni rady 
                                        %zalozene na vzorkovaci frekvenci
  t = tt(1:len);
  if (a==1)
    x = data';
  else
    x = data;
  end 
end
%Adaptace dat pokud jsou zadana ve sloupci
if (a == 2) 
  x=data(:,1);          % namerena data
  t=data(:,2);          % casova osa
end
if (b == 2)
  t=data(:,1);
  x=data(:,2); 
end

%Posunuti dat na osu
 
 x = x-mean(x);       % mean(x) - stredni hodnota
 bpf=x;

%Derivace dat za ucelem detekce hran R piku
 dff = diff(bpf);  % derivovana data

% Umocneni dat - suda funkce - vsechny hodnoty jsou kladne a zvysi se
% rozdily
 sqr = dff.*dff;   % umocnena derivovana data
 
% Uprava poctu vzorku 
 len = len-1; % pocet rozdilu je vzdy o jeden vzorek mene

% Integrace derivovanych dat pres okno d 
 d=[1 1 1 1 1 1 1]; % podle literatury vhodna velikost okna pri Fs = 256
 if (samp_freq>=256) % pro vetsi vzorkovaci frekvenci okno rozsirime
   d = [ones(1,round(7*samp_freq/256))]; 
 end
 
% Integrace
% Integrace m 2 sti:
% Vpoet klouzavho prmru ( postup z prikladu v napovede Matlabu
runavg = filter(d,1,sqr);
% Aplikace medianoveho filtru
mdfint = medfilt1(runavg,10);

%Pri aplikaci filtru dojde ke vzniku zpozdeni, s ohledem na to ze vysledky
%budeme vyuzivat k detekci na puvodnim signalu, musime toto zpozdeni
%odstranit
 delay = ceil(length(d)/2);         %vypocet zpozdeni filtru
 mdfint = mdfint(delay:length(mdfint)); %posun dat o zpozdeni

 %-------------------------------------------------------------------------
 %Detekce piku
 
 %Nejprve je treba urcit maximalni hodnotu ke ktere bude vztazena
 %pozadovana mez
 % Pri testovani realnych dat se v nekterych vzorcich objevily pulsy s
 % podstatne vyssi strmosti amplitudy nez mela realna ekg data, proto hledame
 % prumer z 20 nejvyssich amplitud v integrovanych datech. Pokud je mezi
 % mezi maximy nalezen rozdl vt ne 100 % dochz ke snen
 % amplitud maxim na polovinu.
 maxmdfint = mdfint;
 for i = 1:20
     if ( i > 1)
         while (max(maxval)/ max(maxmdfint)) > 2 % detekce rozdlu mexi maximy
             [tmpmax, tmpindex] = max(maxval);
             maxval(tmpindex) = tmpmax/2; % redukce amplitud maxim na polovinu
         end
     end
     [maxval(i),index(i)] = max(maxmdfint); %nalezeni maximalni hodnoty
        maxmdfint(index(i)-10:index(i)+10) = 0;             % nalezene maximum vynulujeme 
        
        
 end;
 max_h = mean(maxval);

 % nalezeni oblasti dat ktere splnuji podminku zadane mezni hodnoty
 poss_reg = mdfint>(thresh*max_h);

 % v oblastech nad mezni hranici nalezneme okrajove hodnoty mezi nimi
 % budeme hledat maximum
 left  = find(diff([0 poss_reg'])==1); % leve hranice
 right = find(diff([poss_reg' 0])==-1); % prave hranice
 
 % Projdeme zvolene oblasti mezi nalezenymi hranicemi a v teto oblasti
 % nalezneme maxima a minima. Maximum v naleze oblasti povazujeme za R pik,
 % minimum za S vlnu
 
 for(i=1:length(left))
     % pozice a amplituda maxima v dane oblasti
    [maxval(i) maxloc(i)] = max( bpf(left(i):right(i)) ); 
    % pozice a amplituda minima v dane oblasti
    [minval(i) minloc(i)] = min( bpf(left(i):right(i)) );
    %nalezeny index se vztahuje k oblasti ve ktere hledame, pro urceni
    %pozice v celem intervalu dat musime pricist levy kraj aktualne
    %prohledavane oblasti
    
    maxloc(i) = maxloc(i)-1+left(i); 
    minloc(i) = minloc(i)-1+left(i); 
 end

 % Naplneni vystupnich promenych:
 %R piky:
 R_index = maxloc; % pozice maxim - R piku
 R_t   = t(maxloc);% casy maxim - R piku
 R_amp = maxval;   % amplitudy R piku 
 % S vlny:
 S_amp = minval;   % amplituda S vlny
 S_t   = t(minloc);% casy S vlny

% Pri zpracovani realnych dat doslo u nekolika vzorku ke spatnemu pripojeni
% elektrod. S vlna pak predchazela R piku a jeji amplituda byla vetsi.
% Pouzitelnost takovych dat lze zajistit jejich inverzi
 
 if (minloc(length(minloc))<maxloc(length(minloc))) 
  R_t   = t(minloc);
  R_amp = minval;
  S_t   = t(maxloc);
  S_amp = maxval;
 end

% Vypocet variability
hrv  = diff(R_t); 
% Velikost prechodu R piku
resp = R_amp-S_amp; 

% Zobrazeni dat a detailu detekce
if (testmode~=0) % Graf se zobrazi pouze pokud je testmode = 1
figure(3)
hold off
subplot(4,1,1)
plot(t,x);hold on;plot(t,bpf,'r')
title('Zdrojov zznam EKG')
ylabel('Amp [mV]')
xlabel('t[s]')
hold off;
subplot(4,1,2)
plot(t(1:length(mdfint)),mdfint);hold on;
plot(t,max(mdfint)*bpf/(2*max(bpf)),'r')
plot(t(left),mdfint(left),'og')
plot(t(right),mdfint(right),'om')
title('Integrovan data s vyznaenm detekovanch okraj pulsu')
ylabel('Int EKG [mV]')
xlabel('t[s]')
hold off;
subplot(4,1,3)
plot(t,bpf,'r');hold on;
plot(R_t,R_amp,'+k');
plot(S_t,S_amp,'+g');
title('EKG s vyznaenm detekovanch R-vln (ern) a S-vln ( zelen)')
ylabel('Amp [mV]')
xlabel('t[s]')
hold off;
subplot(4,1,4)
hold off
plot(R_t(1:length(hrv)),hrv,'r+')
%plot(R_t,resp,'g')
hold on
title('RR intervaly')
ylabel('RR [s]')
xlabel('t[s]')
hold off

end
